@@ -1,3 +1,4 @@
+/* SPDX-License-Identifier: GPL-2.0 */
 #ifndef _LINUX_FIRMWARE_H
 #define _LINUX_FIRMWARE_H
 
@@ -11,7 +12,6 @@
 struct firmware {
 	size_t size;
 	const u8 *data;
-	struct page **pages;
 
 	/* firmware loader private fields */
 	void *priv;
@@ -36,11 +36,15 @@ struct builtin_fw {
 
 #define DECLARE_BUILTIN_FIRMWARE_SIZE(name, blob, size)			     \
 	static const struct builtin_fw __fw_concat(__builtin_fw,__COUNTER__) \
-	__used __section(.builtin_fw) = { name, blob, size }
+	__used __section(".builtin_fw") = { name, blob, size }
 
 #if defined(CONFIG_FW_LOADER) || (defined(CONFIG_FW_LOADER_MODULE) && defined(MODULE))
 int request_firmware(const struct firmware **fw, const char *name,
 		     struct device *device);
+int firmware_request_nowarn(const struct firmware **fw, const char *name,
+			    struct device *device);
+int firmware_request_platform(const struct firmware **fw, const char *name,
+			      struct device *device);
 int request_firmware_nowait(
 	struct module *module, bool uevent,
 	const char *name, struct device *device, gfp_t gfp, void *context,
@@ -49,6 +53,9 @@ int request_firmware_direct(const struct firmware **fw, const char *name,
 			    struct device *device);
 int request_firmware_into_buf(const struct firmware **firmware_p,
 	const char *name, struct device *device, void *buf, size_t size);
+int request_partial_firmware_into_buf(const struct firmware **firmware_p,
+				      const char *name, struct device *device,
+				      void *buf, size_t size, size_t offset);
 
 void release_firmware(const struct firmware *fw);
 #else
@@ -58,6 +65,21 @@ static inline int request_firmware(const struct firmware **fw,
 {
 	return -EINVAL;
 }
+
+static inline int firmware_request_nowarn(const struct firmware **fw,
+					  const char *name,
+					  struct device *device)
+{
+	return -EINVAL;
+}
+
+static inline int firmware_request_platform(const struct firmware **fw,
+					    const char *name,
+					    struct device *device)
+{
+	return -EINVAL;
+}
+
 static inline int request_firmware_nowait(
 	struct module *module, bool uevent,
 	const char *name, struct device *device, gfp_t gfp, void *context,
@@ -83,7 +105,19 @@ static inline int request_firmware_into_buf(const struct firmware **firmware_p,
 	return -EINVAL;
 }
 
+static inline int request_partial_firmware_into_buf
+					(const struct firmware **firmware_p,
+					 const char *name,
+					 struct device *device,
+					 void *buf, size_t size, size_t offset)
+{
+	return -EINVAL;
+}
+
 #endif
+
+int firmware_request_cache(struct device *device, const char *name);
+
 #ifndef _LINUX_LIBRE_FIRMWARE_H
 #define _LINUX_LIBRE_FIRMWARE_H
 
@@ -105,18 +139,30 @@ report_missing_free_firmware(const char *name, const char *what)
 	return -ENOENT;
 }
 static inline int
-reject_firmware(const struct firmware **fw,
-		const char *name, struct device *device)
+firmware_reject_nowarn(const struct firmware **fw,
+		       const char *name, struct device *device)
 {
 	const struct firmware *xfw = NULL;
-	int retval, retval0;
-	retval0 = report_missing_free_firmware(dev_name(device), NULL);
-	retval = request_firmware(&xfw, NONFREE_FIRMWARE, device);
+	int retval, retval0 = -ENOENT;
+	if (fw) *fw = xfw;
+	retval = firmware_request_nowarn(&xfw, NONFREE_FIRMWARE, device);
 	if (!retval) {
 		release_firmware(xfw);
 		retval = retval0;
 	}
-	return -EINVAL;
+	return retval;
+}
+static inline int
+reject_firmware(const struct firmware **fw,
+		const char *name, struct device *device)
+{
+	int retval, retval0;
+	retval0 = report_missing_free_firmware(dev_name(device), NULL);
+	retval = firmware_reject_nowarn(fw, name, device);
+	if (!retval) {
+		retval = retval0;
+	}
+	return retval;
 }
 static inline int
 maybe_reject_firmware(const struct firmware **fw,
@@ -168,6 +214,54 @@ maybe_reject_firmware_nowait(struct module *module, int uevent,
 		return request_firmware_nowait(module, uevent, name,
 					       device, gfp, context, cont);
 }
+static inline int
+reject_firmware_into_buf(const struct firmware **firmware_p, const char *name,
+			 struct device *device, void *buf, size_t size)
+{
+	const struct firmware *xfw = NULL;
+	int retval, retval0;
+	if (firmware_p) *firmware_p = xfw;
+	retval0 = report_missing_free_firmware(dev_name(device), NULL);
+	retval = request_firmware_into_buf(&xfw, NONFREE_FIRMWARE, device, buf, size);
+	if (!retval) {
+		release_firmware(xfw);
+		retval = retval0;
+	}
+	return retval;
+}
+static inline int
+maybe_reject_firmware_into_buf(const struct firmware **firmware_p, const char *name,
+			       struct device *device, void *buf, size_t size)
+{
+	if (is_nonfree_firmware(name))
+		return reject_firmware_into_buf(firmware_p, name, device, buf, size);
+	else
+		return request_firmware_into_buf(firmware_p, name, device, buf, size);
+}
+static inline int
+reject_partial_firmware_into_buf(const struct firmware **firmware_p, const char *name,
+				 struct device *device, void *buf, size_t size, size_t offset)
+{
+	const struct firmware *xfw = NULL;
+	int retval, retval0;
+	if (firmware_p) *firmware_p = xfw;
+	retval0 = report_missing_free_firmware(dev_name(device), NULL);
+	retval = request_partial_firmware_into_buf(&xfw, NONFREE_FIRMWARE, device, buf, size, offset);
+	if (!retval) {
+		release_firmware(xfw);
+		retval = retval0;
+	}
+	return retval;
+}
+static inline int
+maybe_reject_partial_firmware_into_buf(const struct firmware **firmware_p, const char *name,
+				       struct device *device, void *buf, size_t size, size_t offset)
+{
+	if (is_nonfree_firmware(name))
+		return reject_partial_firmware_into_buf(firmware_p, name, device, buf, size, offset);
+	else
+		return request_partial_firmware_into_buf(firmware_p, name, device, buf, size, offset);
+}
 
 #endif /* _LINUX_LIBRE_FIRMWARE_H */
 
